module.exports = {
achievements: require(__dirname + '/build/achievements.json'),
achievements_extended: require(__dirname + '/build/achievements_extended.json'),
challenges: require(__dirname + '/build/challenges.json'),
game_types: require(__dirname + '/build/game_types.json'),
guild_achievements: require(__dirname + '/build/guild_achievements.json'),
languages: require(__dirname + '/build/languages.json'),
maps: require(__dirname + '/build/maps.json'),
modes: require(__dirname + '/build/modes.json'),
pet_xp: require(__dirname + '/build/pet_xp.json'),
quests: require(__dirname + '/build/quests.json'),
skyblock_bazaar: require(__dirname + '/build/skyblock_bazaar.json'),
skyblock_collections: require(__dirname + '/build/skyblock_collections.json'),
skyblock_items: require(__dirname + '/build/skyblock_items.json'),
skyblock_skills: require(__dirname + '/build/skyblock_skills.json')
};