"""Player Stats."""
from .arcade import Arcade
from .arena import Arena
from .battleground import Battleground
from .bedwars import Bedwars
from .bedwars import BedwarsGame
from .build_battle import BuildBattle
from .duels import Duels
from .gingerbread import GingerBread
from .housing import Housing
from .hunger_games import HungerGames
from .legacy import Legacy
from .mcgo import MCGO
from .paintball import Paintball
from .pit import Pit
from .quake import Quake
from .skyclash import SkyClash
from .skywars import Skywars
from .tnt_games import TNTGames
from .uhc import UHC
from .vampirez import VampireZ
from .walls import Walls
from .walls3 import Walls3

__all__ = [
    "Arcade",
    "Arena",
    "Battleground",
    "Bedwars",
    "BuildBattle",
    "Duels",
    "GingerBread",
    "Housing",
    "HungerGames",
    "Legacy",
    "MCGO",
    "Paintball",
    "Pit",
    "Quake",
    "SkyClash",
    "TNTGames",
    "UHC",
    "VampireZ",
    "Walls",
    "Walls3",
    "BedwarsGame",
    "Skywars",
]
